<?php
namespace UBRIR;

use MongoDB\Driver\Server;

class ServerConnector
{
    const ServerURI_TEST = 'https://91.208.121.201:7443/Exec';
    const ServerURI_NORMAL = 'https://twpg.ubrr.ru:8443/Exec';

    public static function CheckOrderStatus(CMS_Payment $cmsModule)
    {
        $shopId = $cmsModule->getShopId();
        $TWPG_OrderId = $cmsModule->getTWPG_OrderId();
        $TWPG_SessionId = $cmsModule->getTWPG_SessionId();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                  <TKKPG>
                    <Request>
                      <Operation>GetOrderStatus</Operation>
                      <Language>RU</Language>
                      <Order>
                        <Merchant>' . $shopId . '</Merchant>
                        <OrderID>' . $TWPG_OrderId . '</OrderID>
                      </Order>
                      <SessionID>' . $TWPG_SessionId . '</SessionID>
                    </Request>
                  </TKKPG>';
        //return $this->xml_extract_status_result($this->send_xml($data), $status);

        return self::SendXML($xml, $cmsModule);
    }

    public static function OrderStatusDecode($orderStatus)
    {
        switch ($orderStatus) {
            case 'APPROVED':
                return 'Одобрен (оплата прошла успешно)';
                break;
            case 'CANCELED':
                return 'Отменен (клиент прервал выполнение операции)';
                break;
            case 'DECLINED':
                return 'Отказ в оплате';
                break;
            case 'REVERSED':
                return 'Реверсирован';
                break;
            case 'REFUNDED':
                return 'Осуществлен возврат товара';
                break;
            case 'PREAUTH-APPROVED':
                return 'Выполнена предавторизация';
                break;
            case 'EXPIRED':
                return 'Истек срок действия заказа';
                break;
            case 'ON-PAYMENT':
                return 'На оплате';
                break;
            default:
                return $orderStatus;
                break;
        }
    }

    public static function PrepareToPay(CMS_Payment $cmsModule)
    {
        $Amount = $cmsModule->getAmount();
        $Merchant = $cmsModule->getShopId();

        $ApproveURL = $cmsModule->getApproveURL();
        $CancelURL = $cmsModule->getApproveURL();
        $DeclineURL = $cmsModule->getApproveURL();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                  <TKKPG>
                    <Request>
                      <Operation>CreateOrder</Operation>
                      <Language>RU</Language>
                      <Order>
                        <OrderType>Purchase</OrderType>
                        <Merchant>' . $Merchant . '</Merchant>
                        <Amount>' . $Amount . '</Amount>
                        <Currency>643</Currency>
                        <Description>Товары</Description>
                        <ApproveURL>' . $ApproveURL . '</ApproveURL>
                        <CancelURL>' . $CancelURL . '</CancelURL>
                        <DeclineURL>' . $DeclineURL . '</DeclineURL>
                      </Order>
                    </Request>
                  </TKKPG>';

        return self::SendXML($xml, $cmsModule);
    }

    private static function SendXML($xml, CMS_Payment $cmsModule)
    {
        $caFile = $cmsModule->getCACertificateFile();
        $certFile = $cmsModule->getPublicCertificateFile();
        $keyFile = $cmsModule->getPrivateCertificateFile();
        $privateCertPass = $cmsModule->getPrivateCertificatePass();

        $WORK_MODE = $cmsModule->getWorkMode();

        if ($WORK_MODE == 'NORMAL') {
            $serverURI = self::ServerURI_NORMAL;
        } else {
            $serverURI = self::ServerURI_TEST;
        }

        $cmsModule->Log('=> Передаем данные серверу (URL = ' . $serverURI . ')');
        $cmsModule->Log($xml);

        $ch = curl_init($serverURI);    // initialize curl handle
        curl_setopt($ch, CURLOPT_POST, 1);

        if ($WORK_MODE == CMS_Payment::WorkMode_Normal) {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        } else {
            //DevMode
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        }

        curl_setopt($ch, CURLOPT_SSLCERT, $certFile);
        curl_setopt($ch, CURLOPT_CAINFO, $caFile);

        // this with CURLOPT_SSLKEYPASSWD
        curl_setopt($ch, CURLOPT_SSLKEY, $keyFile);
        curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $privateCertPass);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        // curl_setopt($ch, CURLOPT_STDERR, $stdout);

        $xmlResponse = curl_exec($ch);

        $response = null;
        if ($xmlResponse === false) {
            $response = new DataContainer();
            $response->AddError('Не удалось связаться с платежным шлюзом');
            $response->AddError(curl_error($ch), 'CURL_ERROR');
            $response->AddError(curl_errno($ch), 'CURL_ERROR_NUMBER');

            $cmsModule->LogErrors($response);
        } else {
            $cmsModule->Log('<= Получили ответ');
            $cmsModule->Log($xmlResponse);
            $objXML = simplexml_load_string($xmlResponse);

            $arKeyValueXML = ServerConnector::XML2Array($xmlResponse);

            $response = new DataContainer();
            $response->SaveXMLData($xmlResponse, $objXML, $arKeyValueXML);

            //Анализируем ответ на наличие ошибок
            $response = self::AnalyzeServerResponse($response);
        }
        curl_close($ch);

        return $response;
    }

    private static function AnalyzeServerResponse(DataContainer $serverResponse)
    {
        $serverResponse->AddError('Проверки не реализованы', 'ERROR');
        $serverResponse->setSuccess();

        return $serverResponse;
    }

    private static function normalizeSimpleXML($obj, $lastKey, &$arKeyValue)
    {
        $data = $obj;
        if (is_object($data)) {
            $data = get_object_vars($data);
        }
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $lastKey = $key;

                if (is_object($value) || is_array($value)) {
                    self::normalizeSimpleXML($value, $lastKey, $arKeyValue);
                } else {
                    //Добавление в результирующий массив (копия здесь для того чтобы немного сократить рекурсию
                    if (isset($arKeyValue[$lastKey])) {
                        if (!is_array($arKeyValue[$lastKey])) {
                            $oldValue = $arKeyValue[$lastKey];
                            $arKeyValue[$lastKey] = Array($oldValue);
                        }
                        $arKeyValue[$lastKey][] = $value;
                    } else {
                        $arKeyValue[$lastKey] = $value;
                    }

                }
            }
        } else {
            //Сложно сказать когда это сработает теперь :)
            $arKeyValue[$lastKey] = $data;
        }
    }

    /**
     * Сплющиваем XML до одномерного массива. При дублировании названий нод - конвертируем результат в массив
     * @param $xmlString
     * @return array
     */
    public static function XML2Array($xmlString)
    {
        $arKeyValue = Array();
        $lastKey = '';

        self::normalizeSimpleXML(simplexml_load_string($xmlString), $lastKey, $arKeyValue);

        return $arKeyValue;
    }

    public static function XML2NestedArray($xml)
    {
        $result = Array();

        function normalizeSimpleXML($obj, &$result)
        {
            $data = $obj;
            if (is_object($data)) {
                $data = get_object_vars($data);
            }
            if (is_array($data)) {
                foreach ($data as $key => $value) {
                    $res = null;

                    normalizeSimpleXML($value, $res);
                    if (($key == '@attributes') && ($key)) {
                        $result = $res;
                    } else {
                        $result[strtolower($key)] = $res;
                    }
                }
            } else {
                $result = $data;
            }
        }

        normalizeSimpleXML(simplexml_load_string($xml), $result);

        return $result;
    }
}