<?php

/**
 * Support:
 * https://opencartforum.com/user/3463-shoputils/
 * http://opencart.shoputils.ru/?route=information/contact
 *
 */
include_once 'classes/UBRIR_OpenCart.php';

class ControllerExtensionPaymentUbrr extends Controller
{
    private $error = array();
    private $version = '2.0';

    public function index()
    {
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $this->load->model('setting/setting');
            $this->model_setting_setting->editSetting('ubrr', $this->request->post);
            $this->session->data['success'] = sprintf($this->language->get('text_success'), $this->language->get('heading_title'));
            $this->response->redirect($this->makeUrl('extension/extension', 'type=payment'));
        }

        $this->CheckDBStructure();

        $data = $this->LoadLocalizationData();
        $this->document->setTitle($this->language->get('heading_title'));

        $data['action'] = $this->makeUrl('extension/payment/ubrr');
        $data['cancel'] = $this->makeUrl('extension/extension', 'type=payment');
        $data['terms'] = '';

        $data = array_merge($data, $this->ParseFormData());
        $data['ajax_response'] = $this->CheckAjaxRequests();

        if (empty($data['ajax_response'])) { //Небольшая оптимизация, если возрвращаем ajax - то нам нет нужды перерисовывать остальные части админки
            $data['header'] = $this->load->controller('common/header');
            $data['column_left'] = $this->load->controller('common/column_left');
            $data['footer'] = $this->load->controller('common/footer');

            $cmsModule = new UBRIR\OpenCart();
            $data['debug_log'] = $cmsModule->GetDebugLogText();
            $data['action_clean_debug_log'] = $this->makeUrl('extension/payment/ubrr/clean_debug_log');
        } else {
            $data['header'] = '';
            $data['column_left'] = '';
            $data['footer'] = '';
        }

        $this->response->setOutput($this->load->view('extension/payment/ubrr', $data));
    }

    public function clean_debug_log()
    {
        $cmsModule = new UBRIR\OpenCart();
        $data['debug_log'] = $cmsModule->CleanDebugLog();

        $returnUrl = $this->makeUrl('extension/payment/ubrr');
        echo '<meta http-equiv="Refresh" content="0; url=' . $returnUrl . '"/>';
    }

    protected function CheckDBStructure()
    {
        //TODO: можно хранить в конфиге что структура была проверена - он все равно каждый раз собирается из БД, чтобы не делать дополнительные запросы
        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "twpg_orders` (`shoporderid` VARCHAR(11), `OrderID`
                int(11) NOT NULL,`SessionID` VARCHAR(40) NOT NULL) ENGINE=InnoDB DEFAULT CHARSET=utf8");
    }

    protected function CheckAjaxRequests()
    {
        include_once 'UbrrClass.php';
        include_once 'UbrrOpencart.php';

        $ajax_response = '';

        if (isset($_REQUEST['order_id'])) {
            $order_id = intval($_REQUEST['order_id']);
            if ($order_id <= 0) {
                return 'Неверный номер заказа';
            }
            $twpg_order_row = \UbrrOpencart::getTwpgData($this->db, $order_id);
        }
        $shop_id = $this->config->get('ubrr_entry_merchant');
        $sert = $this->config->get('ubrr_sslkeypass');

        $ajaxType = '';
        if (isset($_GET['ajax_type']))
            $ajaxType = $_GET['ajax_type'];

        if ($ajaxType !== '') {
            //Делаем проверки перед выполенением запросов
            if (empty($shop_id))
                return 'Незаполнен или не сохранен параметр [Номер Ретейлера], необходимый для идентификации в системе';
            //default value = '4100104'

            if (empty($sert))
                return 'Незаполнен или не сохранен параметр [Пароль к сертификату], необходимый для идентификации в системе';
            //$sert = '123456';

            switch ($ajaxType) {
                case 'shoporderidforstatus': {
                    $bankHandler = new Ubrir(array(                                                                                                                 // для статуса
                        'shopId' => $shop_id,
                        'order_id' => $order_id,
                        'sert' => $sert,
                        'twpg_order_id' => $twpg_order_row['twpg_order_id'],
                        'twpg_session_id' => $twpg_order_row['twpg_session_id']
                    ));
                    $ajax_response = '<div class="ubr_s">Статус заказа - ' . $bankHandler->check_status() . '</div>';
                    return $ajax_response;
                }
                    break;

                case 'shoporderidfordetailstatus': {
                    $bankHandler = new Ubrir(array(                                                                                                                 // для статуса
                        'shopId' => $shop_id,
                        'order_id' => $order_id,
                        'sert' => $sert,
                        'twpg_order_id' => $twpg_order_row['twpg_order_id'],
                        'twpg_session_id' => $twpg_order_row['twpg_session_id']
                    ));
                    $ajax_response = '<div class="ubr_s">Статус заказа - ' . $bankHandler->detailed_status() . '</div>';
                    return $ajax_response;
                }
                    break;

                case 'recresult': {
                    $bankHandler = new Ubrir(array(                                                                                                                 // для статуса
                        'shopId' => $shop_id

                    ));
                    $ajax_response = '<div class="ubr_s">Статус заказа - ' . $bankHandler->reconcile() . '</div>';
                    return $ajax_response;
                }
                    break;

                case 'recresult': {
                    $bankHandler = new Ubrir(array(                                                                                                                 // для статуса
                        'shopId' => $shop_id
                    ));
                    $ajax_response = '<div class="ubr_s">Статус заказа - ' . $bankHandler->reconcile() . '</div>';
                    return $ajax_response;
                }
                    break;

                case 'journal': {
                    $bankHandler = new Ubrir(array(                                                                                                                 // для статуса
                        'shopId' => $shop_id

                    ));
                    $ajax_response = '<div class="ubr_s">Статус заказа - ' . $bankHandler->extract_journal() . '</div>';
                    return $ajax_response;
                }
                    break;
            }
        }

        return $ajax_response;
    }

    protected
    function LoadLocalizationData()
    {
        $data = Array();

        $this->load->language('extension/payment/ubrr');
        $this->load->model('localisation/language');
        $this->load->model('localisation/order_status');

        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

        $data['lang_heading_title'] = $this->language->get('heading_title');

        $data['lang_save'] = $this->language->get('save');
        $data['lang_cancel'] = $this->language->get('cancel');

        $data['lang_tab_general'] = $this->language->get('tab_general');
        $data['lang_status'] = $this->language->get('status');
        $data['lang_enabled'] = $this->language->get('enabled');
        $data['lang_disabled'] = $this->language->get('disabled');

        $data['lang_ubrr_entry_merchant'] = $this->language->get('ubrr_entry_merchant');

        $data['lang_tab_order_status'] = $this->language->get('tab_order_status');
        $data['lang_entry_order_status'] = $this->language->get('entry_order_status');

        $data['lang_text_enabled'] = $this->language->get('text_enabled');
        $data['lang_text_disabled'] = $this->language->get('text_disabled');
        $data['lang_merchant'] = $this->language->get('merchant');
        $data['lang_sslkeypass'] = $this->language->get('sslkeypass');
        $data['lang_order_description'] = $this->language->get('order_description');

        $data['lang_tab_debug'] = $this->language->get('tab_debug');

        //Загрузка названия для цепочки навигации
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->makeUrl('common/dashboard')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_payment'),
            'href' => $this->makeUrl('extension/extension', 'type=payment')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->makeUrl('extension/payment/ubrr')
        );

        return $data;
    }

    protected function ParseFormData()
    {
        $data = Array();

        if (isset($this->request->post['ubrr_sslkeypass'])) {
            $data['ubrr_sslkeypass'] = $this->request->post['ubrr_sslkeypass'];
        } else {
            $data['ubrr_sslkeypass'] = $this->config->get('ubrr_sslkeypass');
        }

        if (isset($this->request->post['ubrr_status'])) {
            $data['ubrr_status'] = $this->request->post['ubrr_status'];
        } else {
            $data['ubrr_status'] = $this->config->get('ubrr_status');
        }

        $data['error_ubrr_entry_merchant'] = ''; //Наполнить если есть ошибка
        if (isset($this->request->post['ubrr_entry_merchant'])) {
            $data['ubrr_entry_merchant'] = $this->request->post['ubrr_entry_merchant'];
        } else {
            $data['ubrr_entry_merchant'] = $this->config->get('ubrr_entry_merchant');
        }

        if (isset($this->request->post['ubrr_order_status_id'])) {
            $data['ubrr_order_status_id'] = $this->request->post['ubrr_order_status_id'];
        } else {
            $data['ubrr_order_status_id'] = $this->config->get('ubrr_order_status_id');
        }

        return $data;
    }

    protected
    function validate()
    {
        if (!$this->validatePermission()) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        return !$this->error;
    }

    protected
    function _updateData($keys, $info = array())
    {
        $data = array();
        foreach ($keys as $key) {
            if (isset($this->request->post[$key])) {
                $data[$key] = $this->request->post[$key];
            } elseif ($this->config->get($key)) {
                $data[$key] = $this->config->get($key);
            } elseif (isset($info[$key])) {
                $data[$key] = $info[$key];
            } else {
                $data[$key] = null;
            }
        }
        return $data;
    }

    protected
    function validatePermission()
    {
        return $this->user->hasPermission('modify', 'extension/payment/ubrr');
    }

    protected
    function makeUrl($route, $url = '')
    {
        return str_replace('&amp;', '&', $this->url->link($route, $url . '&token=' . $this->session->data['token'], 'SSL'));
    }
}

?>