<?php

class ControllerExtensionPaymentUbrr extends Controller
{
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->language('extension/payment/ubrr');
    }

    public function index()
    {
        $this->load->language('extension/payment/ubrr');

//LoadLanguage
        $data['text_testmode'] = $this->language->get('text_testmode');
        $data['button_confirm'] = $this->language->get('button_confirm');

//LoadParams
        $data['order_id'] = $this->session->data['order_id'];

        $actionURL = $this->url->link('extension/payment/ubrr/form_submit') . '&orderid=' . $data['order_id'];
        $data['action'] = $actionURL;

        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        if ($order_info) {
            $data['currency'] = $order_info['currency_code'];

            $total = $this->currency->format($order_info['total'], $order_info['currency_code'], false, false);
            $data['order_amount'] = $total;
        }

        return $this->load->view('extension/payment/ubrr', $data);
    }

    public function form_submit()
    {
        include_once('classes/UBRIR_OpenCart.php');

        $cmsModule = new UBRIR\OpenCart();

        $shopId = $this->config->get('ubrr_entry_merchant');
        $certificatePassword = $this->config->get('ubrr_sslkeypass');

        $cmsModule->ServerConnectionConfig($shopId, $certificatePassword);

        //Готовимся оплачивать
        $orderId = intval($_POST['orderid']);
        $orderAmount = floatval($_POST['orderamount']);
        $callbackURL = $this->url->link('extension/payment/ubrr/callback');
        $response = $cmsModule->SetupAndPrepareToPay($orderId, $orderAmount, $callbackURL);

        //Подготовка прошла успешно
        if ($response->isSuccess()) {
            $PaymentURL = $response->getFieldValue('Url');
            $TWPG_OrderId = $response->getFieldValue('OrderId');
            $SessionId = $response->getFieldValue('SessionId');

            UBRIR\OpenCart::SaveTWPGOrder($this->db, $orderId, $TWPG_OrderId, $SessionId);

            echo '<meta http-equiv="Refresh" content="0; url=' . $PaymentURL . '?orderid=' . $TWPG_OrderId . '&sessionid=' . $SessionId . '">';
        } else {
            $resultData = new UBRIR\ResultViewer();

            $extraData['public_errors'] = Array();
            if ($response !== null) {
                if ($response->GetMode() === 'TEST') {
                    $arMessages = $response->getMessages();


                    foreach ($arMessages as $key => $list) {
                        foreach ($list as $message) {
                            $resultData->AddExtraMessage($message);
                        }
                    }
                }
            }

            $this->ShowPaymentFailurePage($resultData, 'Ошибка подготовки платежа');
        }
    }

    public function callback()
    {
        include_once('classes/UBRIR_OpenCart.php');

        $openCartModule = new UBRIR\OpenCart();
        $requestDataContainer = $openCartModule->HandleCallback();

        if ($requestDataContainer->isSuccess()) {
            $orderStatus = $requestDataContainer->getFieldValue('OrderStatus');

            $TWPG_OrderID = $requestDataContainer->getFieldValue('OrderId');
            //Получить номер локального заказа по банковскому
            $dbResult = UBRIR\OpenCart::GetTWPGDataByUBRIROrderID($this->db, $TWPG_OrderID);
            if ($dbResult === false) {
                $this->ShowPaymentFailurePage('Не найден заказ с платежным идентификатором = ' . $TWPG_OrderID);
                die;
            }

            $OrderId = $dbResult['order_id'];
            $SessionId = $dbResult['twpg_session_id'];

            $resultData = new UBRIR\ResultViewer();
            $resultData
                ->SetPaymentTime($requestDataContainer->getFieldValue('date'))
                ->SetShopOrderId($OrderId)
                ->SetTWPG_OrderId($TWPG_OrderID)
                ->SetCardPAN($requestDataContainer->getFieldValue('PAN'))
                ->SetCardOwner($requestDataContainer->getFieldValue('Name'))
                ->SetPaymentAmount($requestDataContainer->getFieldValue('PurchaseAmountScr'));

            switch ($orderStatus) {
                case 'APPROVED': {
                    $this->load->model('checkout/order');

                    $shopId = $this->config->get('ubrr_entry_merchant');
                    $certificatePassword = $this->config->get('ubrr_sslkeypass');
                    $openCartModule->ServerConnectionConfig($shopId, $certificatePassword);

                    $checkOrderStatusResponse = $openCartModule->SetupAndCheckOrderStatus($TWPG_OrderID, $SessionId);


                    if ($checkOrderStatusResponse->isSuccess()) {
                        $this->load->model('checkout/order');

                        $ubrr_order_status_id = $this->config->get('ubrr_order_status_id');
                        $this->model_checkout_order->addOrderHistory($OrderId, $ubrr_order_status_id);

                        self::SendOrderSuccessEmail($OrderId);

                        $message = $requestDataContainer->getFieldValue('ResponseDescription');

                        $this->ShowPaymentSuccessPage($resultData, $message);
                    } else {
                        $bankResponseDescription = $checkOrderStatusResponse->getFieldValue('ResponseDescription');
                        $resultData->AddExtraMessage($bankResponseDescription, 'Сообщение');

                        $this->ShowPaymentFailurePage($resultData, 'Ошибка проверки оплаты заказа №' . $OrderId);
                    }
                }
                    break;
                case 'CANCELED': {
                    $resultData->SetMessageBaseOnResponse($requestDataContainer);

                    $canceledMessage = UBRIR\TypeEnums::GetLabel(UBRIR\TypeEnums::OrderStatus_CANCELED, 'ru');

                    $this->ShowPaymentFailurePage($resultData, $canceledMessage);
                }
                    break;
                case 'DECLINED': {
                    $resultData->SetMessageBaseOnResponse($requestDataContainer);

                    $this->ShowPaymentFailurePage($resultData);
                }
                    break;
                default: {
                    $this->ShowPaymentFailurePage($resultData, 'Неверный статус оплаты');
                }
                    break;
            }
        } else {
            $this->ShowPaymentFailurePage(new UBRIR\ResultViewer(), 'Неверный формат ответа');
        }
    }

    public function test_new()
    {
        include_once('classes/UBRIR_OpenCart.php');
        echo UBRIR\TypeEnums::GetLabel(UBRIR\TypeEnums::OrderStatus_DECLINED, 'ru');
    }

    public function test_empty_error()
    {
        include_once('classes/UBRIR_OpenCart.php');

        $resultData = new UBRIR\ResultViewer();

        $this->ShowPaymentFailurePage($resultData, 'Ошибка');
    }

    public function test_failure()
    {
        include_once('classes/UBRIR_OpenCart.php');

        $resultData = new UBRIR\ResultViewer();
        $resultData
            ->SetPaymentTime('01.02.2017 в 17:12')
            ->SetShopOrderId(1)
            ->SetTWPG_OrderId(5037)
            ->SetCardPAN('5106&nbsp;21**&nbsp;****&nbsp;9412')
            ->SetCardOwner('IVANOV PETR')
            ->SetPaymentAmount(2000);

        $resultData->AddExtraMessage('Тут могут быть расшифровки ошибок');

        $this->ShowPaymentFailurePage($resultData, 'Ошибка проверки оплаты заказа №' . 5037);
    }

    public function test_success()
    {
        include_once('classes/UBRIR_OpenCart.php');

        $resultData = new UBRIR\ResultViewer();
        $resultData
            ->SetPaymentTime('01.02.2017 в 17:12')
            ->SetShopOrderId(1)
            ->SetTWPG_OrderId(5037)
            ->SetCardPAN('5106&nbsp;21**&nbsp;****&nbsp;9412')
            ->SetCardOwner('IVANOV PETR')
            ->SetPaymentAmount(2000);

        $this->ShowPaymentSuccessPage($resultData, 'Оплачен');
    }

    public function email_tpl_test()
    {
        $order_id = $this->request->get['order_id'];

        $data = self::GetOrderInfo($order_id);
        if ($data === false) {
            echo 'Заказ не найден!';
        } else {

            $data['email_title'] = '';
            $data['email_body'] = '';

            $eMailHtml = $this->load->view('extension/payment/ubrr_email_order', $data);
            echo $eMailHtml;
        }
    }

    private function GetOrderInfo($order_id)
    {
        $this->load->language('account/order');
        $this->load->model('account/order');

        $data = Array();

        $order_info = $this->model_account_order->getOrder($order_id);

        if ($order_info) {
            $data['heading_title'] = $this->language->get('text_order');

            $data['text_order_detail'] = $this->language->get('text_order_detail');
            $data['text_invoice_no'] = $this->language->get('text_invoice_no');
            $data['text_order_id'] = $this->language->get('text_order_id');
            $data['text_date_added'] = $this->language->get('text_date_added');
            $data['text_shipping_method'] = $this->language->get('text_shipping_method');
            $data['text_shipping_address'] = $this->language->get('text_shipping_address');
            $data['text_payment_method'] = $this->language->get('text_payment_method');
            $data['text_payment_address'] = $this->language->get('text_payment_address');
            $data['text_history'] = $this->language->get('text_history');
            $data['text_comment'] = $this->language->get('text_comment');
            $data['text_no_results'] = $this->language->get('text_no_results');

            $data['column_name'] = $this->language->get('column_name');
            $data['column_model'] = $this->language->get('column_model');
            $data['column_quantity'] = $this->language->get('column_quantity');
            $data['column_price'] = $this->language->get('column_price');
            $data['column_total'] = $this->language->get('column_total');
            $data['column_action'] = $this->language->get('column_action');
            $data['column_date_added'] = $this->language->get('column_date_added');
            $data['column_status'] = $this->language->get('column_status');
            $data['column_comment'] = $this->language->get('column_comment');

            $data['button_reorder'] = $this->language->get('button_reorder');
            $data['button_return'] = $this->language->get('button_return');
            $data['button_continue'] = $this->language->get('button_continue');

            if (isset($this->session->data['error'])) {
                $data['error_warning'] = $this->session->data['error'];

                unset($this->session->data['error']);
            } else {
                $data['error_warning'] = '';
            }

            if (isset($this->session->data['success'])) {
                $data['success'] = $this->session->data['success'];

                unset($this->session->data['success']);
            } else {
                $data['success'] = '';
            }

            if ($order_info['invoice_no']) {
                $data['invoice_no'] = $order_info['invoice_prefix'] . $order_info['invoice_no'];
            } else {
                $data['invoice_no'] = '';
            }

            $data['order_id'] = $order_id;
            $data['date_added'] = date($this->language->get('date_format_short'), strtotime($order_info['date_added']));

            if ($order_info['payment_address_format']) {
                $format = $order_info['payment_address_format'];
            } else {
                $format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
            }

            $find = array(
                '{firstname}',
                '{lastname}',
                '{company}',
                '{address_1}',
                '{address_2}',
                '{city}',
                '{postcode}',
                '{zone}',
                '{zone_code}',
                '{country}'
            );

            $replace = array(
                'firstname' => $order_info['payment_firstname'],
                'lastname' => $order_info['payment_lastname'],
                'company' => $order_info['payment_company'],
                'address_1' => $order_info['payment_address_1'],
                'address_2' => $order_info['payment_address_2'],
                'city' => $order_info['payment_city'],
                'postcode' => $order_info['payment_postcode'],
                'zone' => $order_info['payment_zone'],
                'zone_code' => $order_info['payment_zone_code'],
                'country' => $order_info['payment_country']
            );

            $data['payment_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));

            $data['payment_method'] = $order_info['payment_method'];

            if ($order_info['shipping_address_format']) {
                $format = $order_info['shipping_address_format'];
            } else {
                $format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
            }

            $find = array(
                '{firstname}',
                '{lastname}',
                '{company}',
                '{address_1}',
                '{address_2}',
                '{city}',
                '{postcode}',
                '{zone}',
                '{zone_code}',
                '{country}'
            );

            $replace = array(
                'firstname' => $order_info['shipping_firstname'],
                'lastname' => $order_info['shipping_lastname'],
                'company' => $order_info['shipping_company'],
                'address_1' => $order_info['shipping_address_1'],
                'address_2' => $order_info['shipping_address_2'],
                'city' => $order_info['shipping_city'],
                'postcode' => $order_info['shipping_postcode'],
                'zone' => $order_info['shipping_zone'],
                'zone_code' => $order_info['shipping_zone_code'],
                'country' => $order_info['shipping_country']
            );

            $data['shipping_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));

            $data['shipping_method'] = $order_info['shipping_method'];

            $this->load->model('catalog/product');
            $this->load->model('tool/upload');

            // Products
            $data['products'] = array();

            $products = $this->model_account_order->getOrderProducts($order_id);

            foreach ($products as $product) {
                $option_data = array();

                $options = $this->model_account_order->getOrderOptions($order_id, $product['order_product_id']);

                foreach ($options as $option) {
                    if ($option['type'] != 'file') {
                        $value = $option['value'];
                    } else {
                        $upload_info = $this->model_tool_upload->getUploadByCode($option['value']);

                        if ($upload_info) {
                            $value = $upload_info['name'];
                        } else {
                            $value = '';
                        }
                    }

                    $option_data[] = array(
                        'name' => $option['name'],
                        'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
                    );
                }

                $product_info = $this->model_catalog_product->getProduct($product['product_id']);

                if ($product_info) {
                    $reorder = $this->url->link('account/order/reorder', 'order_id=' . $order_id . '&order_product_id=' . $product['order_product_id'], true);
                } else {
                    $reorder = '';
                }

                $data['products'][] = array(
                    'name' => $product['name'],
                    'model' => $product['model'],
                    'option' => $option_data,
                    'quantity' => $product['quantity'],
                    'price' => $this->currency->format($product['price'] + ($this->config->get('config_tax') ? $product['tax'] : 0), $order_info['currency_code'], $order_info['currency_value']),
                    'total' => $this->currency->format($product['total'] + ($this->config->get('config_tax') ? ($product['tax'] * $product['quantity']) : 0), $order_info['currency_code'], $order_info['currency_value']),
                    'reorder' => $reorder,
                    'return' => $this->url->link('account/return/add', 'order_id=' . $order_info['order_id'] . '&product_id=' . $product['product_id'], true)
                );
            }

            // Voucher
            $data['vouchers'] = array();

            $vouchers = $this->model_account_order->getOrderVouchers($order_id);

            foreach ($vouchers as $voucher) {
                $data['vouchers'][] = array(
                    'description' => $voucher['description'],
                    'amount' => $this->currency->format($voucher['amount'], $order_info['currency_code'], $order_info['currency_value'])
                );
            }

            // Totals
            $data['totals'] = array();

            $totals = $this->model_account_order->getOrderTotals($order_id);

            foreach ($totals as $total) {
                $data['totals'][] = array(
                    'title' => $total['title'],
                    'text' => $this->currency->format($total['value'], $order_info['currency_code'], $order_info['currency_value']),
                );
            }

            $data['comment'] = nl2br($order_info['comment']);

            // History
            $data['histories'] = array();

            $results = $this->model_account_order->getOrderHistories($order_id);

            foreach ($results as $result) {
                $data['histories'][] = array(
                    'date_added' => date($this->language->get('date_format_short'), strtotime($result['date_added'])),
                    'status' => $result['status'],
                    'comment' => $result['notify'] ? nl2br($result['comment']) : ''
                );
            }

            $data['continue'] = $this->url->link('account/order', '', true);

            $data['column_left'] = $this->load->controller('common/column_left');
            $data['column_right'] = $this->load->controller('common/column_right');
            $data['content_top'] = $this->load->controller('common/content_top');
            $data['content_bottom'] = $this->load->controller('common/content_bottom');
            $data['footer'] = $this->load->controller('common/footer');
            $data['header'] = $this->load->controller('common/header');
        } else {
            return false;
        }

        return $data;
    }

    private function SendOrderSuccessEmail($order_id)
    {
        $order_data = self::GetOrderInfo($order_id);
        if ($order_data === false) {
            echo 'Заказ номер ' . $order_id . ' не найден!';
            return false;
        }

        $mail = new Mail();
        $mail->protocol = $this->config->get('config_mail_protocol');
        $mail->parameter = $this->config->get('config_mail_parameter');
        $mail->smtp_hostname = $this->config->get('config_mail_smtp_hostname');
        $mail->smtp_username = $this->config->get('config_mail_smtp_username');
        $mail->smtp_password = html_entity_decode($this->config->get('config_mail_smtp_password'), ENT_QUOTES, 'UTF-8');
        $mail->smtp_port = $this->config->get('config_mail_smtp_port');
        $mail->smtp_timeout = $this->config->get('config_mail_smtp_timeout');

        $adminEmail = $this->config->get('config_email');
        $storeName = $this->config->get('config_name');
        $subject = 'Оплачен заказ №' . $order_id;
        $data['email_title'] = $subject;
        $data['email_body'] = '';
        $data = array_merge($data, $order_data);

        $eMailHtml = $this->load->view('extension/payment/ubrr_email_order', $data);

        $mail->setTo($adminEmail);
        $mail->setFrom($adminEmail);
        $mail->setSender(html_entity_decode($storeName, ENT_QUOTES, 'UTF-8'));
        $mail->setSubject(html_entity_decode($subject, ENT_QUOTES, 'UTF-8'));

        $mail->setHtml($eMailHtml);
        $mail->send();
    }

    private function ShowPaymentFailurePage(UBRIR\ResultViewer $resultData, $errorMessage = '')
    {
        $continue_link = $this->url->link('checkout/cart');

        $data = $resultData
            ->SetFailureMessage($errorMessage)
            ->SetContinueLink($continue_link)
            ->ViewData();

        $this->response->setOutput($this->load->view('extension/payment/ubrr_payment_status', $data));
    }

    private function ShowPaymentSuccessPage(UBRIR\ResultViewer $resultData, $message = '')
    {
        $continue_link = $this->url->link('checkout/success');

        $data = $resultData
            ->SetSuccessMessage($message)
            ->SetContinueLink($continue_link)
            ->ViewData();

        $this->response->setOutput($this->load->view('extension/payment/ubrr_payment_status', $data));
    }
}